=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-02-22 04:29:54 +0000
+++ bzrlib/osutils.py	2006-04-05 05:43:06 +0000
@@ -583,7 +583,14 @@
 
     os.path.commonprefix (python2.4) has a bad bug that it works just
     on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
-    avoids that problem."""
+    avoids that problem.
+    """
+    if sys.platform != "win32":
+        minlength = 1
+    else:
+        minlength = 3
+    assert len(base) >= minlength, ('Length of base must be equal or exceed the'
+        ' platform minimum length (which is %d)' % minlength)
     rp = abspath(path)
 
     s = []

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-03-31 00:43:21 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-04-05 05:43:06 +0000
@@ -825,6 +825,18 @@
         # trailing slash should be the same.
         self.assertEqual('foo', t.relpath(t.base + 'foo/'))
 
+    def test_relpath_at_root(self):
+        t = self.get_transport()
+        # clone all the way to the top
+        new_transport = t.clone('..')
+        while new_transport.base != t.base:
+            t = new_transport
+            new_transport = t.clone('..')
+        # we must be able to get a relpath below the root
+        self.assertEqual('', t.relpath(t.base))
+        # and a deeper one should work too
+        self.assertEqual('foo/bar', t.relpath(t.base + 'foo/bar'))
+
     def test_abspath(self):
         # smoke test for abspath. Corner cases for backends like unix fs's
         # that have aliasing problems like symlinks should go in backend
@@ -836,6 +848,23 @@
         self.assertEqual(transport.base + 'relpath',
                          transport.abspath('relpath'))
 
+    def test_abspath_at_root(self):
+        t = self.get_transport()
+        # clone all the way to the top
+        new_transport = t.clone('..')
+        while new_transport.base != t.base:
+            t = new_transport
+            new_transport = t.clone('..')
+        # we must be able to get a abspath of the root when we ask for
+        # t.abspath('..') - this due to our choice that clone('..')
+        # should return the root from the root, combined with the desire that
+        # the url from clone('..') and from abspath('..') should be the same.
+        self.assertEqual(t.base, t.abspath('..'))
+        # '' should give us the root
+        self.assertEqual(t.base, t.abspath(''))
+        # and a path should append to the url
+        self.assertEqual(t.base + 'foo', t.abspath('foo'))
+
     def test_iter_files_recursive(self):
         transport = self.get_transport()
         if not transport.listable():

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-04-02 22:42:19 +0000
+++ bzrlib/transport/__init__.py	2006-04-05 05:43:06 +0000
@@ -33,6 +33,7 @@
 import sys
 from unittest import TestSuite
 import urllib
+import urlparse
 
 from bzrlib.trace import mutter, warning
 import bzrlib.errors as errors
@@ -119,6 +120,12 @@
     return result
 
 
+def ensure_netloc(protocol):
+    """Ensure that protocol is setup to be used with urlparse netloc parsing."""
+    if protocol not in urlparse.uses_netloc:
+        urlparse.uses_netloc.append(protocol)
+
+
 class Transport(object):
     """This class encapsulates methods for retrieving or putting a file
     from/to a storage location.

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2006-03-12 15:18:19 +0000
+++ bzrlib/transport/http/__init__.py	2006-04-05 05:43:06 +0000
@@ -143,8 +143,11 @@
         # I'm concerned about when it chooses to strip the last
         # portion of the path, and when it doesn't.
         path = '/'.join(basepath)
-        return urlparse.urlunparse((self._qualified_proto,
+        if path == '':
+            path = '/'
+        result = urlparse.urlunparse((self._qualified_proto,
                                     self._host, path, '', '', ''))
+        return result
 
     def _real_abspath(self, relpath):
         """Produce absolute path, adjusting protocol if needed"""
@@ -340,6 +343,7 @@
 #---------------- test server facilities ----------------
 # TODO: load these only when running tests
 
+
 class WebserverNotAvailable(Exception):
     pass
 

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-03-22 11:45:28 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-04-05 05:43:06 +0000
@@ -31,7 +31,7 @@
                            TransportError, ConnectionError,
                            DependencyNotPresent)
 from bzrlib.trace import mutter
-from bzrlib.transport import Transport
+from bzrlib.transport import ensure_netloc
 from bzrlib.transport.http import HttpTransportBase, extract_auth, HttpServer
 
 try:
@@ -41,6 +41,9 @@
     raise DependencyNotPresent('pycurl', e)
 
 
+ensure_netloc('http+pycurl')
+
+
 class PyCurlTransport(HttpTransportBase):
     """http client transport using pycurl
 

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2006-03-22 12:44:32 +0000
+++ bzrlib/transport/http/_urllib.py	2006-04-05 05:43:06 +0000
@@ -19,11 +19,15 @@
 import bzrlib  # for the version
 from bzrlib.errors import BzrError
 from bzrlib.trace import mutter
+from bzrlib.transport import ensure_netloc
 from bzrlib.transport.http import HttpTransportBase, extract_auth, HttpServer
 from bzrlib.errors import (TransportNotPossible, NoSuchFile,
                            TransportError, ConnectionError)
 
 
+ensure_netloc('http+urllib')
+
+
 class Request(urllib2.Request):
     """Request object for urllib2 that allows the method to be overridden."""
 

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2006-03-31 00:43:21 +0000
+++ bzrlib/transport/local.py	2006-04-05 05:43:06 +0000
@@ -57,11 +57,12 @@
             return LocalTransport(self.abspath(offset))
 
     def abspath(self, relpath):
-        """Return the full url to the given relative URL.
-        This can be supplied with a string or a list
-        """
+        """Return the full url to the given relative URL."""
         assert isinstance(relpath, basestring), (type(relpath), relpath)
-        return pathjoin(self.base, urllib.unquote(relpath))
+        result = normpath(pathjoin(self.base, urllib.unquote(relpath)))
+        #if result[-1] != '/':
+        #    result += '/'
+        return result
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.
@@ -69,9 +70,13 @@
         from bzrlib.osutils import relpath
         if abspath is None:
             abspath = u'.'
-        if abspath.endswith('/'):
+        if len(abspath) > 1 and abspath.endswith('/'):
             abspath = abspath[:-1]
-        return relpath(self.base[:-1], abspath)
+        if self.base == '/':
+            root = '/'
+        else:
+            root = self.base[:-1]
+        return relpath(root, abspath)
 
     def has(self, relpath):
         return os.access(self.abspath(relpath), os.F_OK)

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2006-04-02 22:42:19 +0000
+++ bzrlib/transport/memory.py	2006-04-05 05:43:06 +0000
@@ -64,7 +64,7 @@
 
     def clone(self, offset=None):
         """See Transport.clone()."""
-        if offset is None:
+        if offset is None or offset == '':
             return copy(self)
         segments = offset.split('/')
         cwdsegments = self._cwd.split('/')[:-1]
@@ -86,7 +86,14 @@
 
     def abspath(self, relpath):
         """See Transport.abspath()."""
-        return self.base[:-1] + self._abspath(relpath)[len(self._cwd) - 1:]
+        # while a little slow, this is sufficiently fast to not matter in our
+        # current environment - XXX RBC 20060404 move the clone '..' handling
+        # into here and call abspath from clone
+        temp_t = self.clone(relpath)
+        if temp_t.base.count('/') == 1:
+            return temp_t.base
+        else:
+            return temp_t.base[:-1]
 
     def append(self, relpath, f):
         """See Transport.append()."""

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-03-10 06:29:53 +0000
+++ bzrlib/transport/sftp.py	2006-04-05 05:43:06 +0000
@@ -38,7 +38,7 @@
                            )
 from bzrlib.osutils import pathjoin, fancy_rename
 from bzrlib.trace import mutter, warning, error
-from bzrlib.transport import Transport, Server, urlescape
+from bzrlib.transport import Transport, Server, urlescape, ensure_netloc
 import bzrlib.ui
 
 try:
@@ -53,8 +53,9 @@
     from paramiko.sftp_file import SFTPFile
     from paramiko.sftp_client import SFTPClient
 
-if 'sftp' not in urlparse.uses_netloc:
-    urlparse.uses_netloc.append('sftp')
+
+ensure_netloc('sftp')
+
 
 # don't use prefetch unless paramiko version >= 1.5.2 (there were bugs earlier)
 _default_do_prefetch = False
@@ -267,7 +268,7 @@
         self._parse_url(base)
         base = self._unparse_url()
         if base[-1] != '/':
-            base = base + '/'
+            base += '/'
         super(SFTPTransport, self).__init__(base)
         if clone_from is None:
             self._sftp_connect()
@@ -611,7 +612,6 @@
             netloc = '%s@%s' % (urllib.quote(self._username), netloc)
         if self._port is not None:
             netloc = '%s:%d' % (netloc, self._port)
-
         return urlparse.urlunparse(('sftp', netloc, path, '', '', ''))
 
     def _split_url(self, url):

